@@ -19,6 +19,7 @@ from .typing import type_in
 from .jsonschema import ActionJsonSchema, supported_types
 from .jsonnet import ActionJsonnet
 from .optionals import (
+    dump_preserve_order_support,
     import_jsonnet,
     argcomplete_support,
     import_argcomplete,
@@ -36,6 +37,7 @@ from .actions import (
     _ActionConfigLoad,
     _find_action,
     _is_action_value_list,
+    filter_default_actions,
 )
 from .util import (
     namespace_to_dict,
@@ -61,6 +63,18 @@ from .util import (
 __all__ = ['ArgumentParser']
 
 
+default_dump_yaml_kwargs = {
+    'default_flow_style': False,
+    'allow_unicode': True,
+    'sort_keys': False if dump_preserve_order_support else True,
+}
+
+default_dump_json_kwargs = {
+    'ensure_ascii': False,
+    'sort_keys': False if dump_preserve_order_support else True,
+}
+
+
 class _ActionsContainer(argparse._ActionsContainer):
     """Extension of argparse._ActionsContainer to support additional functionalities."""
 
@@ -79,11 +93,20 @@ class _ActionsContainer(argparse._ActionsContainer):
         Args:
             enable_path: Whether to try parsing path/subconfig when argument is a complex type.
         """
+        parser = self.parser if hasattr(self, 'parser') else self  # type: ignore
+        if 'action' in kwargs and isinstance(kwargs['action'], ActionParser):
+            if kwargs['action']._parser == parser:
+                raise ValueError('Parser cannot be added as a subparser of itself.')
+            return ActionParser._move_parser_actions(parser, args, kwargs)
         if 'type' in kwargs:
             if type_in(kwargs['type'], supported_types) or \
                (inspect.isclass(kwargs['type']) and not _issubclass(kwargs['type'], (str, int, float, Enum, Path))):
+                if 'action' in kwargs:
+                    raise ValueError('Type hints as type does not allow providing an action')
                 kwargs['action'] = ActionJsonSchema(annotation=kwargs.pop('type'), enable_path=enable_path)
             elif _issubclass(kwargs['type'], Enum):
+                if 'action' in kwargs:
+                    raise ValueError('Enum as type does not allow providing an action')
                 kwargs['action'] = ActionEnum(enum=kwargs['type'])
         action = super().add_argument(*args, **kwargs)
         if not hasattr(action, 'completer') and action.type is not None:
@@ -97,7 +120,6 @@ class _ActionsContainer(argparse._ActionsContainer):
         for key in meta_keys:
             if key in action.dest:
                 raise ValueError('Argument with destination name "'+key+'" not allowed.')
-        parser = self.parser if hasattr(self, 'parser') else self  # type: ignore
         if action.help is None and \
            issubclass(parser.formatter_class, DefaultHelpFormatter):
             action.help = ' '
@@ -105,10 +127,6 @@ class _ActionsContainer(argparse._ActionsContainer):
             parser.required_args.add(action.dest)
             action._required = True  # type: ignore
             action.required = False
-        if isinstance(action, ActionParser):
-            if action._parser == self:
-                raise ValueError('Parser cannot be added as a subparser of itself.')
-            ActionParser._set_inner_parser_prefix(self, action.dest, action)
         return action
 
 
@@ -158,7 +176,10 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             default_env: Set the default value on whether to parse environment variables.
             default_meta: Set the default value on whether to include metadata in config objects.
         """
-        kwargs['formatter_class'] = formatter_class
+        class FormatterClass(formatter_class):  # type: ignore
+            _parser = self
+
+        kwargs['formatter_class'] = FormatterClass
         super().__init__(*args, **kwargs)
         if self.groups is None:
             self.groups = {}
@@ -180,6 +201,8 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             raise ValueError('The only accepted values for parser_mode are {"yaml", "jsonnet"}.')
         if parser_mode == 'jsonnet':
             import_jsonnet('parser_mode=jsonnet')
+        self.dump_yaml_kwargs = dict(default_dump_yaml_kwargs)
+        self.dump_json_kwargs = dict(default_dump_json_kwargs)
 
 
     ## Parsing methods ##
@@ -198,18 +221,10 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             namespace = Namespace()
 
         if caller == 'argcomplete':
-            namespace = _dict_to_flat_namespace(self._merge_config(self.get_defaults(nested=False), namespace))
+            namespace = _dict_to_flat_namespace(self._merge_config(self.get_defaults(nested=False, skip_check=True), namespace))
 
         try:
             namespace, args = self._parse_known_args(args, namespace)
-            if len(args) > 0:
-                for action in self._actions:
-                    if isinstance(action, ActionParser):
-                        ns, args = action._parser.parse_known_args(args)
-                        for key, val in vars(ns).items():
-                            setattr(namespace, key, val)
-                        if len(args) == 0:
-                            break
         except (ArgumentError, ParserError):
             err = sys.exc_info()[1]
             self.error(str(err))
@@ -252,7 +267,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             _ActionSubCommands.handle_subcommands(self, cfg, env=env, defaults=defaults, fail_no_subcommand=fail_no_subcommand)
 
         if nested:
-            cfg = _flat_namespace_to_dict(dict_to_namespace(cfg))
+            cfg = _flat_namespace_to_dict(_dict_to_flat_namespace(cfg))
 
         if cfg_base is not None:
             if isinstance(cfg_base, Namespace):
@@ -264,7 +279,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             cfg = self._merge_config(cfg, cfg_env)
 
         elif defaults:
-            cfg = self._merge_config(cfg, self.get_defaults(nested=nested))
+            cfg = self._merge_config(cfg, self.get_defaults(nested=nested, skip_check=True))
 
         if not (with_meta or (with_meta is None and self._default_meta)):
             cfg = strip_meta(cfg)
@@ -414,13 +429,14 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             if env is None:
                 env = dict(os.environ)
             cfg = {}  # type: ignore
-            for action in self._actions:
+            actions = filter_default_actions(self._actions)
+            for action in actions:
                 env_var = _get_env_var(self, action)
                 if env_var in env and isinstance(action, ActionConfigFile):
                     namespace = _dict_to_flat_namespace(cfg)
                     ActionConfigFile._apply_config(self, namespace, action.dest, env[env_var])
                     cfg = vars(namespace)
-            for action in self._actions:
+            for action in actions:
                 env_var = _get_env_var(self, action)
                 if env_var in env and isinstance(action, _ActionSubCommands):
                     env_val = env[env_var]
@@ -429,19 +445,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                         pcfg = action._name_parser_map[env_val].parse_env(env=env, defaults=defaults, nested=False, _skip_logging=True, _skip_check=True)  # type: ignore
                         for k, v in vars(pcfg).items():
                             cfg[subcommand+'.'+k] = v
-            for action in [a for a in self._actions if a.default != SUPPRESS]:
-                if isinstance(action, ActionParser):
-                    subparser_cfg = {}
-                    if defaults:
-                        subparser_cfg = vars(action._parser.get_defaults(nested=False))  # type: ignore
-                    env_var = _get_env_var(self, action)
-                    if env_var in env:
-                        pcfg = self._check_value_key(action, env[env_var], action.dest, cfg)
-                        subparser_cfg.update(vars(_dict_to_flat_namespace(namespace_to_dict(pcfg))))
-                    pcfg = action._parser.parse_env(env=env, defaults=False, nested=False, with_meta=with_meta, _skip_logging=True, _skip_check=True)  # type: ignore
-                    subparser_cfg.update(namespace_to_dict(pcfg))
-                    cfg.update(subparser_cfg)
-                    continue
+            for action in actions:
                 env_var = _get_env_var(self, action)
                 if env_var in env and not isinstance(action, ActionConfigFile):
                     env_val = env[env_var]
@@ -512,8 +516,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                                            with_meta=with_meta,
                                            _skip_logging=True,
                                            _skip_check=_skip_check,
-                                           _fail_no_subcommand=_fail_no_subcommand,
-                                           _base=_base)
+                                           _fail_no_subcommand=_fail_no_subcommand)
 
         self._logger.info('Parsed %s from path: %s', self.parser_mode, cfg_path)
 
@@ -532,7 +535,6 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         _skip_logging: bool = False,
         _skip_check: bool = False,
         _fail_no_subcommand: bool = True,
-        _base = None,
     ) -> Union[Namespace, Dict[str, Any]]:
         """Parses configuration (yaml or jsonnet) given as a string.
 
@@ -552,7 +554,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             ParserError: If there is a parsing error and error_handler=None.
         """
         try:
-            cfg = self._load_cfg(cfg_str, cfg_path, ext_vars, _base)
+            cfg = self._load_cfg(cfg_str, cfg_path, ext_vars)
 
             parsed_cfg = self._parse_common(
                 cfg=cfg,
@@ -576,7 +578,6 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         cfg_str: str,
         cfg_path: str = '',
         ext_vars: dict = None,
-        base: str = None,
     ) -> Dict[str, Any]:
         """Loads a configuration string (yaml or jsonnet) into a namespace checking all values against the parser.
 
@@ -598,8 +599,6 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         except (yamlParserError, yamlScannerError) as ex:
             raise TypeError('Problems parsing config :: '+str(ex)) from ex
         cfg = namespace_to_dict(_dict_to_flat_namespace(cfg))
-        if base is not None:
-            cfg = {base+'.'+k: v for k, v in cfg.items()}
 
         self._apply_actions(cfg, self._actions)
 
@@ -708,8 +707,6 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                             cfg[action.dest] = [str(p) for p in cfg[action.dest]]
                         else:
                             cfg[action.dest] = str(cfg[action.dest])
-                elif isinstance(action, ActionParser):
-                    cleanup_actions(cfg, action._parser._actions)
                 elif isinstance(action, ActionJsonSchema) and action._annotation is not None and cfg.get(action.dest) is not None:
                     cfg[action.dest] = ActionJsonSchema._adapt_types(cfg[action.dest], action._annotation, action._subschemas, True)
 
@@ -735,11 +732,11 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         if format == 'parser_mode':
             format = 'yaml' if self.parser_mode == 'yaml' else 'json_indented'
         if format == 'yaml':
-            return yaml.dump(cfg, default_flow_style=False, allow_unicode=True)
+            return yaml.dump(cfg, **self.dump_yaml_kwargs)  # type: ignore
         elif format == 'json_indented':
-            return json.dumps(cfg, indent=2, sort_keys=True, ensure_ascii=False)+'\n'
+            return json.dumps(cfg, indent=2, **self.dump_json_kwargs)+'\n'  # type: ignore
         elif format == 'json':
-            return json.dumps(cfg, separators=(',', ':'), sort_keys=True, ensure_ascii=False)
+            return json.dumps(cfg, separators=(',', ':'), **self.dump_json_kwargs)  # type: ignore
         else:
             raise ValueError('Unknown output format "'+str(format)+'".')
 
@@ -801,18 +798,15 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                             if not overwrite and os.path.isfile(val_path()):
                                 raise ValueError('Refusing to overwrite existing file: '+str(val_path))
                             action = _find_action(self, kbase)
-                            if isinstance(action, ActionParser):
-                                replace_keys[key] = val_path
-                                action._parser.save(val, val_path(), branch=action.dest, **save_kwargs)
-                            elif isinstance(action, (ActionJsonSchema, ActionJsonnet)):
+                            if isinstance(action, (ActionJsonSchema, ActionJsonnet, _ActionConfigLoad)):
                                 replace_keys[key] = val_path
                                 val_out = strip_meta(val)
                                 if '__orig__' in val:
                                     val_str = val['__orig__']
                                 elif str(val_path).lower().endswith('.json'):
-                                    val_str = json.dumps(val_out, indent=2, sort_keys=True, ensure_ascii=False)+'\n'
+                                    val_str = json.dumps(val_out, indent=2, **self.dump_json_kwargs)+'\n'
                                 else:
-                                    val_str = yaml.dump(val_out, default_flow_style=False, allow_unicode=True)
+                                    val_str = yaml.dump(val_out, **self.dump_yaml_kwargs)
                                 with open(val_path(), 'w') as f:
                                     f.write(val_str)
                         else:
@@ -850,6 +844,17 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             self.set_defaults(kwargs)
 
 
+    def _get_default_config_file(self):
+        default_config_files = []  # type: List[str]
+        for pattern in self.default_config_files:
+            default_config_files += glob.glob(os.path.expanduser(pattern))
+        if len(default_config_files) > 0:
+            try:
+                return Path(default_config_files[0], mode=get_config_read_mode())
+            except TypeError:
+                pass
+
+
     def get_default(self, dest:str):
         """Gets a single default value for the given destination key.
 
@@ -860,16 +865,20 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             KeyError: If key not defined in the parser.
         """
         action = _find_action(self, dest)
-        if action is None:
+        if action is None or action.default == SUPPRESS or action.dest == SUPPRESS:
             raise KeyError('No action for destination key "'+dest+'" to get its default.')
-        return action.default
+        default_config_file = self._get_default_config_file()
+        if default_config_file is None:
+            return action.default
+        return getattr(self.get_defaults(), action.dest)
 
 
-    def get_defaults(self, nested:bool=True) -> Namespace:
+    def get_defaults(self, nested:bool=True, skip_check:bool=False) -> Namespace:
         """Returns a namespace with all default values.
 
         Args:
             nested: Whether the namespace should be nested.
+            skip_check: Whether to skip check if configuration is valid.
 
         Returns:
             An object with all default values as attributes.
@@ -877,22 +886,21 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         cfg = {}
         for action in self._actions:
             if action.default != SUPPRESS and action.dest != SUPPRESS:
-                if isinstance(action, ActionParser):
-                    cfg.update(namespace_to_dict(action._parser.get_defaults(nested=False)))  # type: ignore
-                else:
-                    cfg[action.dest] = action.default
+                cfg[action.dest] = action.default
 
         cfg = namespace_to_dict(_dict_to_flat_namespace(cfg))
 
         self._logger.info('Loaded default values from parser.')
 
-        default_config_files = []  # type: List[str]
-        for pattern in self.default_config_files:
-            default_config_files += glob.glob(os.path.expanduser(pattern))
-        if len(default_config_files) > 0:
-            default_config_file = Path(default_config_files[0], mode=get_config_read_mode())
+        default_config_file = self._get_default_config_file()
+        if default_config_file is not None:
             with change_to_path_dir(default_config_file):
                 cfg_file = self._load_cfg(default_config_file.get_content())
+                if not skip_check:
+                    try:
+                        self.check_config(cfg_file, skip_required=True)
+                    except (TypeError, KeyError) as ex:
+                        raise ParserError('Problem in default config file "'+str(default_config_file)+'" :: '+ex.args[0]) from ex
             cfg = self._merge_config(cfg_file, cfg)
             cfg['__default_config__'] = default_config_file
             self._logger.info('Parsed configuration from default path: %s', str(default_config_file))
@@ -917,6 +925,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
     def check_config(
         self, cfg: Union[Namespace, Dict[str, Any]],
         skip_none: bool = True,
+        skip_required: bool = False,
         branch: str = None,
     ):
         """Checks that the content of a given configuration object conforms with the parser.
@@ -942,8 +951,8 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                 dct = dct[key]
             return dct
 
-        def check_required(cfg):
-            for reqkey in self.required_args:
+        def check_required(cfg, parser):
+            for reqkey in parser.required_args:
                 try:
                     val = get_key_value(cfg, reqkey)
                     if val is None:
@@ -972,11 +981,12 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                     raise KeyError('No action for key "'+kbase+'" to check its value.')
 
         try:
-            check_required(cfg)
+            if not skip_required:
+                check_required(cfg, self)
             check_values(cfg)
         except (TypeError, KeyError) as ex:
             prefix = 'Configuration check failed :: '
-            message = str(ex)
+            message = ex.args[0]
             if prefix not in message:
                 message = prefix+message
             raise type(ex)(message) from ex
@@ -1052,21 +1062,27 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         return cfg_files
 
 
+    def format_help(self):
+        if len(self._default_config_files) > 0:
+            defaults = namespace_to_dict(self.get_defaults())
+            note = 'no existing default config file found.'
+            if '__default_config__' in defaults:
+                note = 'default values below will be ones overridden by the contents of: '+str(defaults['__default_config__'])
+                self.formatter_class.defaults = vars(_dict_to_flat_namespace(defaults))
+            group = self._default_config_files_group
+            group.description = str(self._default_config_files) + ', Note: '+note
+        help_str = super().format_help()
+        if hasattr(self.formatter_class, 'defaults'):
+            delattr(self.formatter_class, 'defaults')
+        return help_str
+
+
     def _apply_actions(self, cfg, actions):
         """Runs _check_value_key on actions present in flat config dict."""
         for action in actions:
-            if isinstance(action, ActionParser):
-                self._apply_actions(cfg, action._parser._actions)
             if action.dest in cfg:
                 value = self._check_value_key(action, cfg[action.dest], action.dest, cfg)
-                if isinstance(action, ActionParser):
-                    value = namespace_to_dict(_dict_to_flat_namespace(namespace_to_dict(value)))
-                    if '__path__' in value:
-                        value[action.dest+'.__path__'] = value.pop('__path__')
-                    del cfg[action.dest]
-                    cfg.update(value)
-                else:
-                    cfg[action.dest] = value
+                cfg[action.dest] = value
 
 
     @staticmethod
@@ -1138,6 +1154,8 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                         value[k] = action.type(v)
             except (TypeError, ValueError) as ex:
                 raise TypeError('Parser key "'+str(key)+'": '+str(ex)) from ex
+        if isinstance(value, Namespace):
+            value = namespace_to_dict(value)
         return value
 
 
@@ -1187,12 +1205,14 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             raise ValueError('default_config_files has to be None or List[str].')
 
         if len(self._default_config_files) > 0:
-            group_title = 'default config file locations'
-            group = next((g for g in self._action_groups if g.title == group_title), None)
-            if group is None:
+            if not hasattr(self, '_default_config_files_group'):
+                group_title = 'default config file locations'
                 group = _ArgumentGroup(self, title=group_title)
                 self._action_groups = [group] + self._action_groups  # type: ignore
-            group.description = str(self._default_config_files)
+                self._default_config_files_group = group
+        elif hasattr(self, '_default_config_files_group'):
+            self._action_groups = [g for g in self._action_groups if g != self._default_config_files_group]
+            delattr(self, '_default_config_files_group')
 
 
     @property
@@ -1214,8 +1234,6 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             self._default_env = default_env
         else:
             raise ValueError('default_env has to be a boolean.')
-        if _issubclass(self.formatter_class, DefaultHelpFormatter):
-            setattr(self.formatter_class, '_default_env', default_env)
 
 
     @property
@@ -1260,5 +1278,3 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             self._env_prefix = env_prefix
         else:
             raise ValueError('env_prefix has to be a string or None.')
-        if _issubclass(self.formatter_class, DefaultHelpFormatter):
-            setattr(self.formatter_class, '_env_prefix', env_prefix)
