@@ -3,6 +3,7 @@
 import os
 import sys
 import locale
+import platform
 from importlib.util import find_spec
 
 
@@ -43,6 +44,11 @@ else:
     jsonschemaValidationError = None
 
 
+dump_preserve_order_support = True
+if sys.version_info.minor < 6 or platform.python_implementation() != 'CPython':
+    dump_preserve_order_support = False
+
+
 def import_jsonschema(importer):
     try:
         import jsonschema
