@@ -4,6 +4,7 @@ import json
 import pathlib
 from enum import Enum
 from io import StringIO
+from contextlib import redirect_stdout
 from jsonargparse_tests.base import *
 from jsonargparse import ArgumentParser
 
@@ -26,11 +27,6 @@ class SimpleActionsTests(unittest.TestCase):
         self.assertEqual(True,  parser.parse_args(['--no_bools.def_true=no']).bools.def_true)
         self.assertRaises(ParserError, lambda: parser.parse_args(['--bools.def_true nope']))
 
-        self.assertEqual(True,  parser.parse_env({'APP_BOOLS__DEF_FALSE': 'true'}).bools.def_false)
-        self.assertEqual(True,  parser.parse_env({'APP_BOOLS__DEF_FALSE': 'yes'}).bools.def_false)
-        self.assertEqual(False, parser.parse_env({'APP_BOOLS__DEF_TRUE': 'false'}).bools.def_true)
-        self.assertEqual(False, parser.parse_env({'APP_BOOLS__DEF_TRUE': 'no'}).bools.def_true)
-
         parser = ArgumentParser()
         parser.add_argument('--val', action=ActionYesNo)
         self.assertEqual(True,  parser.parse_args(['--val']).val)
@@ -45,6 +41,18 @@ class SimpleActionsTests(unittest.TestCase):
         self.assertRaises(ValueError, lambda: parser.add_argument('--val', nargs='?', action=ActionYesNo(no_prefix=None)))
 
 
+    def test_ActionYesNo_parse_env(self):
+        parser = example_parser()
+        self.assertEqual(True,  parser.parse_env({'APP_BOOLS__DEF_FALSE': 'true'}).bools.def_false)
+        self.assertEqual(True,  parser.parse_env({'APP_BOOLS__DEF_FALSE': 'yes'}).bools.def_false)
+        self.assertEqual(False, parser.parse_env({'APP_BOOLS__DEF_TRUE': 'false'}).bools.def_true)
+        self.assertEqual(False, parser.parse_env({'APP_BOOLS__DEF_TRUE': 'no'}).bools.def_true)
+
+        parser = ArgumentParser(default_env=True, env_prefix='APP')
+        parser.add_argument('--op', action=ActionYesNo, default=False)
+        self.assertEqual(True, parser.parse_env({'APP_OP': 'true'}).op)
+
+
     def test_ActionYesNo_old_bool(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--val', nargs=1, action=ActionYesNo(no_prefix=None))
@@ -77,7 +85,6 @@ class SimpleActionsTests(unittest.TestCase):
         cfg = parser.parse_args(['--enum=C'], with_meta=False)
         self.assertEqual('enum: C\n', parser.dump(cfg))
 
-        os.environ['COLUMNS'] = '150'
         help_str = StringIO()
         parser.print_help(help_str)
         self.assertIn('Description (type: MyEnum, default: C)', help_str.getvalue())
@@ -92,6 +99,7 @@ class SimpleActionsTests(unittest.TestCase):
 
         self.assertRaises(ValueError, lambda: ActionEnum())
         self.assertRaises(ValueError, lambda: ActionEnum(enum=object))
+        self.assertRaises(ValueError, lambda: parser.add_argument('--bad', type=MyEnum, action=True))
 
 
     def test_ActionOperators(self):
@@ -288,10 +296,14 @@ class ActionParserTests(TempDirTestCase):
         self.assertEqual('opt2_arg', cfg.inner2.opt2)
         self.assertEqual('opt3_arg', cfg.inner2.inner3.opt3)
 
-        ## Check invalid initializations
-        self.assertRaises(ValueError, lambda: parser.add_argument('--op1', action=ActionParser))
-        self.assertRaises(ValueError, lambda: ActionParser())
-        self.assertRaises(ValueError, lambda: ActionParser(parser=object))
+
+    def test_ActionParser_required(self):
+        p1 = ArgumentParser()
+        p1.add_argument('--op1', required=True)
+        p2 = ArgumentParser(error_handler=None)
+        p2.add_argument('--op2', action=ActionParser(parser=p1))
+        p2.parse_args(['--op2.op1=1'])
+        self.assertRaises(ParserError, lambda: p2.parse_args([]))
 
 
     def test_ActionParser_failures(self):
@@ -299,8 +311,75 @@ class ActionParserTests(TempDirTestCase):
         parser_lv2.add_argument('--op')
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--inner', action=ActionParser(parser=parser_lv2))
-        self.assertRaises(ValueError, lambda: parser.add_argument('--mistake', action=ActionParser(parser=parser)))
+        self.assertRaises(Exception, lambda: parser.add_argument('--bad', action=ActionParser))
+        self.assertRaises(ValueError, lambda: parser.add_argument('--bad', action=ActionParser(parser=parser)))
+        self.assertRaises(ValueError, lambda: parser.add_argument('--bad', type=str, action=ActionParser(ArgumentParser())))
+        self.assertRaises(ValueError, lambda: parser.add_argument('-b', '--bad', action=ActionParser(ArgumentParser())))
         self.assertRaises(ParserError, lambda: parser.parse_args(['--inner=1']))
+        self.assertRaises(ValueError, lambda: ActionParser())
+        self.assertRaises(ValueError, lambda: ActionParser(parser=object))
+
+
+    def test_ActionParser_conflict(self):
+        parser_lv2 = ArgumentParser()
+        parser_lv2.add_argument('--op')
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--inner.op')
+        self.assertRaises(ValueError, lambda: parser.add_argument('--inner', action=ActionParser(parser_lv2)))
+
+
+    def test_ActionParser_nested_dash_names(self):
+        p1 = ArgumentParser(error_handler=None)
+        p1.add_argument('--op1-like')
+
+        p2 = ArgumentParser(error_handler=None)
+        p2.add_argument('--op2-like', action=ActionParser(parser=p1))
+
+        self.assertEqual(p2.parse_args(['--op2-like.op1-like=a']).op2_like.op1_like, 'a')
+
+        p3 = ArgumentParser(error_handler=None)
+        p3.add_argument('--op3', action=ActionParser(parser=p2))
+
+        self.assertEqual(p3.parse_args(['--op3.op2-like.op1-like=b']).op3.op2_like.op1_like, 'b')
+
+
+    def test_ActionParser_action_groups(self):
+        def get_parser_lv2():
+            parser_lv2 = ArgumentParser(description='parser_lv2 description')
+            parser_lv2.add_argument('--a1', help='lv2_a1 help')
+            group_lv2 = parser_lv2.add_argument_group(description='group_lv2 description')
+            group_lv2.add_argument('--a2', help='lv2_a2 help')
+            return parser_lv2
+
+        parser_lv2 = get_parser_lv2()
+        parser = ArgumentParser()
+        parser.add_argument('--lv2', action=ActionParser(parser_lv2))
+
+        out = StringIO()
+        with redirect_stdout(out):
+            parser.print_help()
+        outval = out.getvalue()
+
+        self.assertIn('parser_lv2 description', outval)
+        self.assertIn('group_lv2 description', outval)
+        self.assertIn('--lv2.a1 A1', outval)
+
+        parser_lv2 = get_parser_lv2()
+        parser = ArgumentParser()
+        parser.add_argument(
+            '--lv2',
+            title='ActionParser title',
+            description='ActionParser description',
+            action=ActionParser(parser_lv2),
+        )
+
+        out = StringIO()
+        with redirect_stdout(out):
+            parser.print_help()
+        outval = out.getvalue()
+
+        self.assertIn('ActionParser title', outval)
+        self.assertIn('ActionParser description', outval)
 
 
 if __name__ == '__main__':
