@@ -1,11 +1,15 @@
 #!/usr/bin/env python3
 
+import sys
 import json
 import yaml
+import platform
 from io import StringIO
 from contextlib import redirect_stdout
 from collections import OrderedDict
+from random import randint, shuffle
 from jsonargparse_tests.base import *
+from jsonargparse.optionals import dump_preserve_order_support
 from jsonargparse.util import meta_keys, _suppress_stderr
 
 
@@ -63,6 +67,17 @@ class ParsersTests(TempDirTestCase):
             parser.env_prefix = lambda: 'invalid'
 
 
+    def test_default_env(self):
+        parser = ArgumentParser()
+        self.assertFalse(parser.default_env)
+        parser.default_env = True
+        self.assertTrue(parser.default_env)
+        parser = ArgumentParser(default_env=True)
+        self.assertTrue(parser.default_env)
+        parser.default_env = False
+        self.assertFalse(parser.default_env)
+
+
     def test_parse_string(self):
         parser = example_parser()
 
@@ -529,6 +544,29 @@ class OutputTests(TempDirTestCase):
         self.assertRaises(ValueError, lambda: parser.dump(cfg, format='invalid'))
 
 
+    @unittest.skipIf(not dump_preserve_order_support,
+                     'Dump preserve order only supported in python>=3.6 and CPython')
+    def test_dump_order(self):
+        args = {}
+        for num in range(50):
+            args[num] = ''.join(chr(randint(97, 122)) for n in range(8))
+
+        parser = ArgumentParser()
+        for num in range(len(args)):
+            parser.add_argument('--'+args[num], default=num)
+
+        cfg = parser.get_defaults()
+        dump = parser.dump(cfg)
+        self.assertEqual(dump, '\n'.join(v+': '+str(n) for n, v in args.items())+'\n')
+
+        rand = list(range(len(args)))
+        shuffle(rand)
+        yaml = '\n'.join(args[n]+': '+str(n) for n in rand)+'\n'
+        cfg = parser.parse_string(yaml)
+        dump = parser.dump(cfg)
+        self.assertEqual(dump, '\n'.join(v+': '+str(n) for n, v in args.items())+'\n')
+
+
     def test_save(self):
         parser = ArgumentParser()
         parser.add_argument('--parser',
@@ -659,17 +697,29 @@ class ConfigFilesTests(TempDirTestCase):
         self.assertEqual('from default config file', cfg.op1)
         self.assertEqual('from parser default', cfg.op2)
 
-        os.environ['COLUMNS'] = '150'
-        out = StringIO()
-        with redirect_stdout(out):
-            parser.print_help()
-        self.assertIn('default config file locations', out.getvalue())
-        self.assertIn(os.path.basename(default_config_file), out.getvalue())
-
         with self.assertRaises(ValueError):
             parser.default_config_files = False
 
 
+    def test_get_default_with_default_config_files(self):
+        default_config_file = os.path.realpath(os.path.join(self.tmpdir, 'example.yaml'))
+        with open(default_config_file, 'w') as output_file:
+            output_file.write('op1: from yaml\n')
+
+        parser = ArgumentParser(default_config_files=[default_config_file])
+        parser.add_argument('--op1', default='from default')
+
+        self.assertEqual(parser.get_default('op1'), 'from yaml')
+
+        with open(default_config_file, 'w') as output_file:
+            output_file.write('op2: v2\n')
+        self.assertRaises(ParserError, lambda: parser.get_default('op1'))
+
+        if os.name == 'posix' and platform.python_implementation() == 'CPython':
+            os.chmod(default_config_file, 0)
+            self.assertEqual(parser.get_default('op1'), 'from default')
+
+
     def test_ActionConfigFile_and_ActionPath(self):
         os.mkdir(os.path.join(self.tmpdir, 'example'))
         rel_yaml_file = os.path.join('..', 'example', 'example.yaml')
@@ -778,6 +828,12 @@ class OtherTests(unittest.TestCase):
         self.assertEqual(cfg, Namespace(op1=1, op2=None, op3=3))
 
 
+    def test_check_config_branch(self):
+        parser = example_parser()
+        cfg = parser.get_defaults()
+        parser.check_config(cfg.lev1, branch='lev1')
+
+
     def test_usage_and_exit_error_handler(self):
         with _suppress_stderr():
             parser = ArgumentParser()
